@@ -1,5 +1,8 @@
 // #define ARMA_64BIT_WORD 1
 #include <RcppArmadillo.h>
+#if defined(_OPENMP)
+  #include <omp.h>
+#endif
 // [[Rcpp::depends(RcppArmadillo)]]
 
 // write RcppExports.R with: 
@@ -86,7 +89,6 @@ arma::mat matL_sparse_rowCumsums(const arma::SpMat<double> & Csq, const int & h,
 	arma::mat out(p, h+1, arma::fill::zeros);
 
   #if defined(_OPENMP)
-  #include <omp.h>
   #pragma omp parallel for num_threads(nthreads)
   #endif
 	
@@ -126,7 +128,10 @@ arma::mat matL_full_rowCumsums(const arma::mat & Csq, const int & h, int nthread
 	int p = Csq.n_rows;
 	arma::mat out(p, h+1, arma::fill::zeros);
 
-	#pragma omp parallel for num_threads(nthreads)
+  #if defined(_OPENMP)
+  #pragma omp parallel for num_threads(nthreads)
+  #endif
+
 	for( int i=0; i<p; i++){
 		
 		int k = 0;
@@ -215,7 +220,10 @@ arma::mat matR_sparse_rowCumsums(const arma::SpMat<double> & Csq, const int & h,
 	int p = Csq.n_rows;
 	arma::mat out(p, h+1, arma::fill::zeros);
 
-	#pragma omp parallel for num_threads(nthreads)
+  #if defined(_OPENMP)
+  #pragma omp parallel for num_threads(nthreads)
+  #endif
+
 	for( int i=0; i<p; i++){
 		
 		int k = 0;
@@ -250,7 +258,10 @@ arma::mat matR_full_rowCumsums(const arma::mat & Csq, const int & h, int nthread
 	int p = Csq.n_rows;
 	arma::mat out(p, h+1, arma::fill::zeros);
 
-	#pragma omp parallel for num_threads(nthreads)
+  #if defined(_OPENMP)
+  #pragma omp parallel for num_threads(nthreads)
+  #endif
+
 	for( int i=0; i<p; i++){
 		
 		int k = 0;
